# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/sec-downloader',
                'doc_host': 'https://Elijas.github.io',
                'git_url': 'https://github.com/Elijas/sec-downloader',
                'lib_path': 'sec_downloader'},
  'syms': { 'sec_downloader.core': {},
            'sec_downloader.requested_filings': { 'sec_downloader.requested_filings.RequestedFilings': ( 'requested_filings.html#requestedfilings',
                                                                                                         'sec_downloader/requested_filings.py'),
                                                  'sec_downloader.requested_filings.RequestedFilings.from_string': ( 'requested_filings.html#requestedfilings.from_string',
                                                                                                                     'sec_downloader/requested_filings.py')},
            'sec_downloader.sec_edgar_downloader_fork': {},
            'sec_downloader.types': {}}}
